﻿"use strict";

var WingtipToys = window.WingtipToys || {};
WingtipToys.Rest = WingtipToys.Rest || {};

WingtipToys.Rest.Libs = function () {

    var create = function (title, description) {
        var deferred = $.ajax({
            url: _spPageContextInfo.webServerRelativeUrl +
                 "/_api/web/lists",
            type: "POST",
            contentType: "application/json;odata=verbose",
            data: JSON.stringify(
                {
                    '__metadata': {
                        'type': 'SP.List'
                    },
                    'Title': title,
                    'Description': description,
                    'BaseTemplate': SP.ListTemplateType.documentLibrary,
                    'AllowContentTypes': true,
                    'ContentTypesEnabled': true
                }),
            headers: {
                "accept": "application/json;odata=verbose",
                "X-RequestDigest": $("#__REQUESTDIGEST").val()
            }
        });

        return deferred.promise()
    },

    remove = function (title) {
        var deferred = $.ajax(
            {
                url: _spPageContextInfo.webServerRelativeUrl +
                    "/_api/web/lists/getByTitle('" + title + "')",
                type: "DELETE",
                headers: {
                    "accept": "application/json;odata=verbose",
                    "X-RequestDigest": $("#__REQUESTDIGEST").val(),
                    "IF-MATCH": "*"
                }
            }
        );

        return deferred.promise()

    },

    upload = function (serverRelativeUrl, filename, file) {

        var deferred = $.Deferred();

        getFileBuffer(file).then(

            function (buffer) {

                $.ajax({
                    url: _spPageContextInfo.webServerRelativeUrl +
                        "/_api/web/GetFolderByServerRelativeUrl('" + serverRelativeUrl + "')/Files" +
                        "/Add(url='" + filename + "', overwrite=true)",
                    type: "POST",
                    data: buffer,
                    processData: false,
                    headers: {
                        "accept": "application/json;odata=verbose",
                        "X-RequestDigest": $("#__REQUESTDIGEST").val(),
                        "content-length": buffer.byteLength
                    },
                    success: function (data) {
                        deferred.resolve(data);
                    },
                    error: function (err) {
                        deferred.reject(err);
                    }
                });
            },
            function (err) {
                deferred.reject(err);
            }
         );

        return deferred.promise();
    },

    download = function (serverRelativeUrl) {
        var deferred = $.ajax(
            {
                url: _spPageContextInfo.webServerRelativeUrl +
                    "/_api/web/GetFolderByServerRelativeUrl('" + serverRelativeUrl + "')/$value",
                type: "GET"
            }
        );

        return deferred.promise()
    },

    setLibraryTemplate = function (title, documentTemplateUrl) {
        var deferred = $.ajax({
            url: _spPageContextInfo.webServerRelativeUrl +
                "/_api/web/lists/getByTitle('" + title + "')",
            type: "POST",
            contentType: "application/json;odata=verbose",
            data: JSON.stringify(
                {
                    '__metadata': {
                        'type': 'SP.List'
                    },
                    'DocumentTemplateUrl': documentTemplateUrl
                }),
            headers: {
                "accept": "application/json;odata=verbose",
                "X-RequestDigest": $("#__REQUESTDIGEST").val(),
                "IF-MATCH": "*",
                "X-Http-Method": "PATCH"
            }
        });

        return deferred.promise()
    },

    setCTypeTemplate = function (contentTypeId, documentTemplateUrl) {
        var deferred = $.ajax({
            url: _spPageContextInfo.webServerRelativeUrl +
                "/_api/web/contentTypes/getById('" + contentTypeId + "')",
            type: "POST",
            contentType: "application/json;odata=verbose",
            data: JSON.stringify(
                {
                    '__metadata': {
                        'type': 'SP.ContentType'
                    },
                    'DocumentTemplate': documentTemplateUrl
                }),
            headers: {
                "accept": "application/json;odata=verbose",
                "X-RequestDigest": $("#__REQUESTDIGEST").val(),
                "IF-MATCH": "*",
                "X-Http-Method": "PATCH"
            }
        });

        return deferred.promise()
    },

    getFileBuffer = function (file) {
        var deferred = $.Deferred();
        var reader = new FileReader();
        reader.onload = function (e) {
            deferred.resolve(e.target.result);
        }
        reader.onerror = function (e) {
            deferred.reject(e.target.error);
        }
        reader.readAsArrayBuffer(file);
        return deferred.promise();
    },
            
    getQueryStringParameter = function (paramToRetrieve) {
            var params = document.URLUnencoded.split("?")[1].split("&");
            var strParams = "";
            for (var i = 0; i < params.length; i = i + 1) {
                var singleParam = params[i].split("=");
                if (singleParam[0] == paramToRetrieve)
                    return decodeURIComponent(singleParam[1]);
            }
        },

    createContentType = function (contentTypeName, description) {

        var deferred = $.ajax({
            url: _spPageContextInfo.webServerRelativeUrl + "/_api/web/contentTypes",
            type: "POST",
            contentType: "application/json;odata=verbose",
            data: JSON.stringify(
                {
                    '__metadata': { 'type': 'SP.ContentType' },
                    'Name': contentTypeName,
                    'Description': description,
                    'Id': {
                        '__metadata': { 'type': 'SP.ContentTypeId' },
                        'StringValue': createCTypeId("0x0101")
                    }
                }),
            headers: {
                "accept": "application/json;odata=verbose",
                "X-RequestDigest": $("#__REQUESTDIGEST").val()
            }
        });

        return deferred.promise()
    },

    getContentType = function (contentTypeId) {
        var deferred = $.ajax(
            {
                url: _spPageContextInfo.webServerRelativeUrl +
                    "/_api/web/contentTypes/getById('" + contentTypeId + "')",
                type: "GET",
                headers: {
                    "accept": "application/json;odata=verbose",
                }
            }
        );

        return deferred.promise()
    },

    bindContentType = function (libraryTitle, contentTypeId) {
        var deferred = $.ajax(
            {
                url: _spPageContextInfo.webServerRelativeUrl +
                    "/_api/web/lists/getByTitle('" + libraryTitle + "')" +
                    "/contentTypes/addAvailableContentType('" + contentTypeId + "')",
                type: "POST",
                contentType: "application/json;odata=verbose",
                headers: {
                    "accept": "application/json;odata=verbose",
                    "X-RequestDigest": $("#__REQUESTDIGEST").val(),
                }
            }
        );

        return deferred.promise()

    },

    createCTypeId = function (parentId) {
        var id = parentId + "00";
        for (var i = 0; i < 9; i++) {
            id += (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
        }
        return id;
    },

    addFieldLink = function (contentTypeId, internalFieldName) {

        //CAN'T GET THIS CALL TO WORK!!
        return

        var scope;
        if (local === true) {
            scope = "";
        }
        else {
            scope = "/_api/site/rootWeb/contentTypes";
        }


        var deferred = $.ajax({
            url: _spPageContextInfo.webServerRelativeUrl + "/_api/web/contentTypes" +
                "/getById('" + contentTypeId + "')/fieldlinks",
            type: "POST",
            contentType: "application/json;odata=verbose",
            data: JSON.stringify(
                {
                    '__metadata': {
                        'type': 'SP.FieldLink'
                    },
                    'Name': internalFieldName
                }),
            headers: {
                "accept": "application/json;odata=verbose",
                "X-RequestDigest": $("#__REQUESTDIGEST").val()
            }
        });

        return deferred.promise()

    };

    return {
        create: create,
        remove: remove,
        upload: upload,
        download: download,
        set_libraryTemplate: setLibraryTemplate,
        set_ctypeTemplate: setCTypeTemplate,
        create_contentType: createContentType,
        bind_contentType: bindContentType,
        add_fieldLink: addFieldLink
    };

}();