﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Microsoft.SharePoint.Client;

namespace ManagedCSOMWeb.Pages
{
    public partial class ErrorScope : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            string appWebUrl = Page.Request["SPAppWebUrl"];
            using (ClientContext ctx = new ClientContext(appWebUrl))
            {
                //Set up error handling
                ExceptionHandlingScope xScope = new ExceptionHandlingScope(ctx);

                using (xScope.StartScope())
                {
                    using (xScope.StartTry())
                    {
                        //Try to update the description of a list named "My List"
                        List myList = ctx.Web.Lists.GetByTitle("My List");
                        myList.Description = "A new description";
                        myList.Update();
                    }
                    using (xScope.StartCatch())
                    {
                        //Fails if the list "My List" does not exist
                        //So, we’ll create a new list
                        ListCreationInformation listCI = new ListCreationInformation();
                        listCI.Title = "My List";
                        listCI.TemplateType = (int)ListTemplateType.GenericList;
                        listCI.QuickLaunchOption = Microsoft.SharePoint.Client.QuickLaunchOptions.On;
                        List list = ctx.Web.Lists.Add(listCI);
                    }
                    using (xScope.StartFinally())
                    {
                        //Try to update the list now if it failed originally
                        List myList = ctx.Web.Lists.GetByTitle("My List");
                        if (myList.Description.Length == 0)
                        {
                            myList.Description = "A new description";
                            myList.Update();
                        }
                    }
                }

                //Execute the entire try-catch as a batch!
                ctx.ExecuteQuery();
            }

        }
    }
}