﻿using System;
using System.IO;
using System.Net;
using System.Xml.Linq;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI.WebControls;

namespace Wingtip
{
    public static class Contacts
    {
        public static string AppWebUrl;

        public static void CreateItem(string LastName, string FirstName, string WorkPhone)
        {
            Uri uri = new Uri(AppWebUrl +
            "/_api/web/lists/getByTitle('Contacts')/items");

            string itemXML = String.Format(@"
                            <entry xmlns='http://www.w3.org/2005/Atom'
                                   xmlns:d='http://schemas.microsoft.com/ado/2007/08/dataservices'
                                   xmlns:m='http://schemas.microsoft.com/ado/2007/08/dataservices/metadata'>
                            <category term='SP.Data.ContactsListItem'
                                   scheme='http://schemas.microsoft.com/ado/2007/08/dataservices/scheme' />
                            <content type='application/xml'>
                                <m:properties>
                                    <d:FirstName>{0}</d:FirstName>
                                    <d:Title>{1}</d:Title>
                                    <d:WorkPhone>{2}</d:WorkPhone>
                                </m:properties>
                            </content>
                            </entry>", FirstName, LastName, WorkPhone);


            HttpWebRequest restRequest = (HttpWebRequest)WebRequest.Create(uri);
            restRequest.Credentials = CredentialCache.DefaultCredentials;
            restRequest.Method = "POST";
            restRequest.Headers["X-RequestDigest"] = GetFormDigest();
            restRequest.Accept = "application/atom+xml";
            restRequest.ContentType = "application/atom+xml";
            restRequest.ContentLength = itemXML.Length;
            StreamWriter sw = new StreamWriter(restRequest.GetRequestStream());
            sw.Write(itemXML);
            sw.Flush();

            HttpWebResponse restResponse = (HttpWebResponse)restRequest.GetResponse();
        }

        public static List<TableRow> ReadAll()
        {
            Uri uri = new Uri(AppWebUrl +
                        "/_api/web/lists/getByTitle('Contacts')/items/" +
                        "?$select=Id,FirstName,Title,WorkPhone" +
                        "&$orderby=Title,FirstName");

            HttpWebRequest restRequest = (HttpWebRequest)WebRequest.Create(uri);
            restRequest.Credentials = CredentialCache.DefaultCredentials;
            restRequest.Method = "GET";

            HttpWebResponse restResponse = (HttpWebResponse)restRequest.GetResponse();
            XDocument atomDoc = XDocument.Load(restResponse.GetResponseStream());
            XNamespace a = "http://www.w3.org/2005/Atom";
            XNamespace d = "http://schemas.microsoft.com/ado/2007/08/dataservices";

            List<TableRow> rows = new List<TableRow>();
            foreach (var entry in atomDoc.Descendants(a + "entry"))
            {
                TableRow r = new TableRow();
                TableCell c1 = new TableCell();
                c1.Text = entry.Descendants(d + "Id").First().Value;
                TableCell c2 = new TableCell();
                c2.Text = entry.Descendants(d + "FirstName").First().Value;
                TableCell c3 = new TableCell();
                c3.Text = entry.Descendants(d + "Title").First().Value;
                TableCell c4 = new TableCell();
                c4.Text = entry.Descendants(d + "WorkPhone").First().Value;
                r.Cells.Add(c1);
                r.Cells.Add(c2);
                r.Cells.Add(c3);
                r.Cells.Add(c4);
                rows.Add(r);
            }

            return rows;
        }

        public static void UpdateItem(string ID, string LastName, string FirstName, string WorkPhone)
        {
            Uri uri = new Uri(AppWebUrl +
            "/_api/web/lists/getByTitle('Contacts')/items(" + ID + ")");

            string itemXML = String.Format(@"
                            <entry xmlns='http://www.w3.org/2005/Atom'
                                   xmlns:d='http://schemas.microsoft.com/ado/2007/08/dataservices'
                                   xmlns:m='http://schemas.microsoft.com/ado/2007/08/dataservices/metadata'>
                            <category term='SP.Data.ContactsListItem'
                                   scheme='http://schemas.microsoft.com/ado/2007/08/dataservices/scheme' />
                            <content type='application/xml'>
                                <m:properties>
                                    <d:FirstName>{0}</d:FirstName>
                                    <d:Title>{1}</d:Title>
                                    <d:WorkPhone>{2}</d:WorkPhone>
                                </m:properties>
                            </content>
                            </entry>", FirstName, LastName, WorkPhone);


            HttpWebRequest restRequest = (HttpWebRequest)WebRequest.Create(uri);
            restRequest.Credentials = CredentialCache.DefaultCredentials;
            restRequest.Method = "POST";
            restRequest.Headers["X-RequestDigest"] = GetFormDigest();
            restRequest.Headers["IF-MATCH"] = "*";
            restRequest.Headers["X-Http-Method"] = "PATCH";
            restRequest.Accept = "application/atom+xml";
            restRequest.ContentType = "application/atom+xml";
            restRequest.ContentLength = itemXML.Length;
            StreamWriter sw = new StreamWriter(restRequest.GetRequestStream());
            sw.Write(itemXML);
            sw.Flush();

            HttpWebResponse restResponse = (HttpWebResponse)restRequest.GetResponse();
        }

        public static void RemoveItem(string ID)
        {
            Uri uri = new Uri(AppWebUrl +
            "/_api/web/lists/getByTitle('Contacts')/items(" + ID + ")");

            HttpWebRequest restRequest = (HttpWebRequest)WebRequest.Create(uri);
            restRequest.Credentials = CredentialCache.DefaultCredentials;
            restRequest.Method = "DELETE";
            restRequest.Headers["X-RequestDigest"] = GetFormDigest();
            restRequest.Headers["IF-MATCH"] = "*";
            restRequest.Accept = "application/atom+xml";
            restRequest.ContentType = "application/atom+xml";

            HttpWebResponse restResponse = (HttpWebResponse)restRequest.GetResponse();
        }

        private static string GetFormDigest()
        {
            Uri uri = new Uri(AppWebUrl + "/_api/contextinfo");

            HttpWebRequest restRequest = (HttpWebRequest)WebRequest.Create(uri);
            restRequest.Credentials = CredentialCache.DefaultCredentials;
            restRequest.Method = "POST";
            restRequest.ContentLength = 0;

            HttpWebResponse restResponse = (HttpWebResponse)restRequest.GetResponse();
            XDocument atomDoc = XDocument.Load(restResponse.GetResponseStream());
            XNamespace d = "http://schemas.microsoft.com/ado/2007/08/dataservices";
            return atomDoc.Descendants(d + "FormDigestValue").First().Value;
        }
    }

 
}